"use strict";
var Promise = require('bluebird');
var url = require('url');
var util = require('util');
var _ = require('lodash');
var fs = require('fs');
var request = require('request-promise');
var cookie = require('cookie');
var path = require('path');
var xmldoc = require('xmldoc');
var Cache_1 = require('./../../utils/Cache');
var UrlHelper_1 = require('./../../utils/UrlHelper');
var consts = require('./../../Consts');
var OnlineUserCredentials = (function () {
    function OnlineUserCredentials(_siteUrl, _authOptions) {
        this._siteUrl = _siteUrl;
        this._authOptions = _authOptions;
    }
    OnlineUserCredentials.prototype.getAuth = function () {
        var parsedUrl = url.parse(this._siteUrl);
        var host = parsedUrl.host;
        var cacheKey = util.format('%s@%s', host, this._authOptions.username);
        var cachedCookie = OnlineUserCredentials.CookieCache.get(cacheKey);
        var spFormsEndPoint = UrlHelper_1.UrlHelper.removeTrailingSlash(parsedUrl.protocol + "//" + host + "/" + consts.FormsPath);
        if (cachedCookie) {
            return Promise.resolve({
                headers: {
                    'Cookie': cachedCookie
                }
            });
        }
        var samlBody = _.template(fs.readFileSync(path.join(__dirname, '..', '..', '..', '..', 'templates', 'online_saml_wsfed.tmpl')).toString())({
            username: this._authOptions.username,
            password: this._authOptions.password,
            endpoint: spFormsEndPoint
        });
        return request
            .post(consts.MSOnlineSts, {
            body: samlBody,
            simple: false,
            rejectUnauthorized: false,
            headers: {
                'Content-Type': 'application/soap+xml; charset=utf-8'
            }
        })
            .then(function (xmlResponse) {
            var xmlDoc = new xmldoc.XmlDocument(xmlResponse);
            var securityTokenResponse = xmlDoc.childNamed('S:Body').firstChild;
            if (securityTokenResponse.name.indexOf('Fault') !== -1) {
                throw new Error(securityTokenResponse.toString());
            }
            var binaryToken = securityTokenResponse.childNamed('wst:RequestedSecurityToken').firstChild.val;
            var now = new Date().getTime();
            var expires = new Date(securityTokenResponse.childNamed('wst:Lifetime').childNamed('wsu:Expires').val).getTime();
            var diff = (expires - now) / 1000;
            var diffSeconds = parseInt(diff.toString(), 10);
            return request
                .post(spFormsEndPoint, {
                headers: {
                    'User-Agent': 'Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0)',
                    'Content-Type': 'application/x-www-form-urlencoded'
                },
                body: binaryToken,
                rejectUnauthorized: false,
                resolveWithFullResponse: true,
                simple: false
            })
                .then(function (data) {
                var fedAuth, rtFa;
                for (var i = 0; i < data.headers['set-cookie'].length; i++) {
                    var headerCookie = data.headers['set-cookie'][i];
                    if (headerCookie.indexOf(consts.FedAuth) !== -1) {
                        fedAuth = cookie.parse(headerCookie)[consts.FedAuth];
                    }
                    if (headerCookie.indexOf(consts.RtFa) !== -1) {
                        rtFa = cookie.parse(headerCookie)[consts.RtFa];
                    }
                }
                var authCookie = 'FedAuth=' + fedAuth + '; rtFa=' + rtFa;
                OnlineUserCredentials.CookieCache.set(cacheKey, authCookie, diffSeconds);
                return {
                    headers: {
                        'Cookie': authCookie
                    }
                };
            });
        });
    };
    ;
    OnlineUserCredentials.CookieCache = new Cache_1.Cache();
    return OnlineUserCredentials;
}());
exports.OnlineUserCredentials = OnlineUserCredentials;
