"use strict";
var _ = require('lodash');
var url = require('url');
var request = require('request-promise');
var ntlm = require('httpntlm').ntlm;
var agent = require('agentkeepalive');
var OnpremiseUserCredentials = (function () {
    function OnpremiseUserCredentials(_siteUrl, _authOptions) {
        this._siteUrl = _siteUrl;
        this._authOptions = _authOptions;
    }
    OnpremiseUserCredentials.prototype.getAuth = function () {
        _.defaults(this._authOptions, { domain: '', workstation: '' });
        var ntlmOptions = _.assign({}, this._authOptions);
        ntlmOptions.url = this._siteUrl;
        var type1msg = ntlm.createType1Message(ntlmOptions);
        var isHttps = url.parse(this._siteUrl).protocol === 'https:';
        var keepaliveAgent = isHttps ? new agent.HttpsAgent() : new agent();
        return request({
            url: this._siteUrl,
            method: 'GET',
            headers: {
                'Connection': 'keep-alive',
                'Authorization': type1msg,
                'Accept': 'application/json;odata=verbose'
            },
            agent: keepaliveAgent,
            resolveWithFullResponse: true,
            simple: false,
            rejectUnauthorized: false,
            strictSSL: false
        })
            .then(function (response) {
            var type2msg = ntlm.parseType2Message(response.headers['www-authenticate']);
            var type3msg = ntlm.createType3Message(type2msg, ntlmOptions);
            return {
                headers: {
                    'Connection': 'Close',
                    'Authorization': type3msg
                },
                options: {
                    strictSSL: false,
                    agent: keepaliveAgent,
                    rejectUnauthorized: false
                }
            };
        });
    };
    ;
    return OnpremiseUserCredentials;
}());
exports.OnpremiseUserCredentials = OnpremiseUserCredentials;
